@@ -2,10 +2,9 @@ __all__ = ['thread',
            'thread_pool',
            'process',
            'PebbleError',
-           'SerializingError',
            'TimeoutError',
            'TaskCancelled']
 
 from .pebble import PebbleError, TimeoutError
-from .process import process, SerializingError, TaskCancelled
+from .process import process, TaskCancelled
 from .thread import thread, thread_pool
